@@ -132,6 +132,7 @@ int no_version;			/* Don't print version to client.  */
 int type = TYPE_A;		/* Default TYPE_A.  */
 int form = FORM_N;		/* Default FORM_N.  */
 int debug;			/* Enable debug mode if 1.  */
+int rfc2577 = 1;		/* Follow suggestions in RFC 2577.  */
 int timeout = 900;		/* Timeout after 15 minutes of inactivity.  */
 int maxtimeout = 7200;		/* Don't allow idle time to be set
 				   beyond 2 hours.  */
@@ -260,6 +261,10 @@ const char doc[] =
   "File Transfer Protocol daemon.";
 #endif
 
+enum {
+  OPT_NONRFC2577 = CHAR_MAX + 1,
+};
+
 static struct argp_option options[] = {
 #define GRID 0
   { "anonymous-only", 'A', NULL, 0,
@@ -292,6 +297,9 @@ static struct argp_option options[] = {
   { "max-timeout", 'T', NULL, 0,
     "reset maximum value of timeout allowed",
     GRID+1 },
+  { "non-rfc2577", OPT_NONRFC2577, NULL, 0,
+    "neglect RFC 2577 by giving info on missing users",
+    GRID+1 },
   { "umask", 'u', "VAL", 0,
     "set default umask",
     GRID+1 },
@@ -416,6 +424,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
 	break;
       }
 
+    case OPT_NONRFC2577:
+      rfc2577 = 0;
+      break;
+
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -756,6 +768,8 @@ bad:
 void
 user (const char *name)
 {
+  int ret;
+
   if (cred.logged_in)
     {
       if (cred.guest || cred.dochroot)
@@ -767,7 +781,8 @@ user (const char *name)
     }
 
   /* Non zero means failed.  */
-  if (auth_user (name, &cred) != 0)
+  ret = auth_user (name, &cred);
+  if (!rfc2577 && ret != 0)
     {
       /* If they gave us a reason.  */
       if (cred.message)
@@ -783,6 +798,17 @@ user (const char *name)
 		cred.remotehost, name);
       return;
     }
+  else if (rfc2577 && ret != 0)
+    cred.delayed_reject = 1;
+  else
+    cred.delayed_reject = 0;
+
+  /* Only messages for anonymous guests are accepted.  */
+  if (rfc2577 && !cred.guest && cred.message)
+    {
+      free (cred.message);
+      cred.message = NULL;
+    }
 
   /* If the server is set to serve anonymous service only
      the request have to come from a guest or a chrooted.  */
@@ -864,6 +890,7 @@ end_login (struct credentials *pcred)
   pcred->remotehost = remotehost;
   pcred->auth_type = atype;
   pcred->logged_in = 0;
+  pcred->delayed_reject = 0;
 }
 
 void
@@ -879,10 +906,19 @@ pass (const char *passwd)
   if (!cred.guest)		/* "ftp" is the only account allowed no password.  */
     {
       /* Try to authenticate the user.  Failed if != 0.  */
-      if (auth_pass (passwd, &cred) != 0)
+      if (auth_pass (passwd, &cred) != 0 || cred.delayed_reject)
 	{
 	  /* Any particular reason?  */
-	  if (cred.message)
+	  if (rfc2577)
+	    {
+	      if (cred.message)
+		{
+		  free (cred.message);
+		  cred.message = NULL;
+		}
+	      reply (530, "Login incorrect.");
+	    }
+	  else if (cred.message)
 	    {
 	      reply (530, "%s", cred.message);
 	      free (cred.message);
@@ -902,6 +938,8 @@ pass (const char *passwd)
 	    {
 	      syslog (LOG_NOTICE, "repeated login failures from %s",
 		      cred.remotehost);
+	      reply (421,
+		     "Service not available, closing control connection.");
 	      exit (EXIT_SUCCESS);
 	    }
 	  return;
@@ -982,7 +1020,10 @@ retrieve (const char *cmd, const char *name)
 	      c = getc (fin);
 	      if (c == EOF)
 		{
-		  perror_reply (550, name);
+		  /* Error code 554 was introduced in RFC 1123.  */
+		  reply (554,
+			 "Action not taken: invalid REST value %jd for %s.",
+			 restart_point, name);
 		  goto done;
 		}
 	      if (c == '\n')
@@ -991,7 +1032,11 @@ retrieve (const char *cmd, const char *name)
 	}
       else if (lseek (fileno (fin), restart_point, SEEK_SET) < 0)
 	{
-	  perror_reply (550, name);
+	  if (errno == EINVAL)
+	    reply (554, "Action not taken: invalid REST value %jd for %s.",
+		   restart_point, name);
+	  else
+	    perror_reply (550, name);
 	  goto done;
 	}
     }
@@ -1053,7 +1098,10 @@ store (const char *name, const char *mode, int unique)
 	      c = getc (fout);
 	      if (c == EOF)
 		{
-		  perror_reply (550, name);
+		  /* Error code 554 was introduced in RFC 1123.  */
+		  reply (554,
+			 "Action not taken: invalid REST value %jd for %s.",
+			 restart_point, name);
 		  goto done;
 		}
 	      if (c == '\n')
@@ -1070,7 +1118,11 @@ store (const char *name, const char *mode, int unique)
 	}
       else if (lseek (fileno (fout), restart_point, SEEK_SET) < 0)
 	{
-	  perror_reply (550, name);
+	  if (errno == EINVAL)
+	    reply (554, "Action not taken: invalid REST value %jd for %s.",
+		   restart_point, name);
+	  else
+	    perror_reply (550, name);
 	  goto done;
 	}
     }
