@@ -126,9 +126,11 @@ int wlen;
  * rsh - remote shell
  */
 int rfd2;
+int end_of_pipe = 0;		/* Used by parent process.  */
 
 char *copyargs (char **);
 void sendsig (int);
+void sigpipe (int);
 void talk (int, sigset_t *, pid_t, int);
 void warning (const char *, ...);
 
@@ -551,7 +553,12 @@ try_connect:
   if (signal (SIGTERM, SIG_IGN) != SIG_IGN)
     signal (SIGTERM, sendsig);
 
-  if (!null_input_option)
+  if (null_input_option)
+    /* Nothing from stdin will be written to the socket,
+     * but we still expect response from the server.
+     */
+    shutdown (rem, SHUT_WR);
+  else
     {
       pid = fork ();
       if (pid < 0)
@@ -602,6 +609,9 @@ talk (int null_input_option, sigset_t * osigs, pid_t pid, int rem)
   int cc, wc;
   fd_set readfrom, ready, rembits;
   char *bp, buf[BUFSIZ];
+#ifdef HAVE_SIGACTION
+  struct sigaction sa;
+#endif
 
   if (!null_input_option && pid == 0)
     {
@@ -649,15 +659,29 @@ talk (int null_input_option, sigset_t * osigs, pid_t pid, int rem)
 	goto reread;
       goto rewrite;
     done:
-      shutdown (rem, 1);
+      shutdown (rem, SHUT_WR);
       exit (EXIT_SUCCESS);
     }
 
 #ifdef HAVE_SIGACTION
   sigprocmask (SIG_SETMASK, osigs, NULL);
-#else
+#else /* !HAVE_SIGACTION */
   sigsetmask (*osigs);
 #endif
+
+  /* The access to SIGPIPE is neeeded to kill the child process
+   * in an orderly fashion, for example when a command line
+   * pipe fails.  Otherwise the child lives on eternally.
+   */
+#ifdef HAVE_SIGACTION
+  sigemptyset (&sa.sa_mask);
+  sa.sa_flags = SA_RESTART;
+  sa.sa_handler = sigpipe;
+  (void) sigaction (SIGPIPE, &sa, NULL);
+#else /* !HAVE_SIGACTION */
+  signal (SIGPIPE, sigpipe);
+#endif
+
   FD_ZERO (&readfrom);
   FD_SET (rfd2, &readfrom);
   FD_SET (rem, &readfrom);
@@ -694,7 +718,7 @@ talk (int null_input_option, sigset_t * osigs, pid_t pid, int rem)
 		FD_CLR (rfd2, &readfrom);
 	    }
 	  else
-	    write (2, buf, cc);
+	    write (STDERR_FILENO, buf, cc);
 	}
       if (FD_ISSET (rem, &ready))
 	{
@@ -717,10 +741,11 @@ talk (int null_input_option, sigset_t * osigs, pid_t pid, int rem)
 		FD_CLR (rem, &readfrom);
 	    }
 	  else
-	    write (1, buf, cc);
+	    write (STDOUT_FILENO, buf, cc);
 	}
     }
-  while (FD_ISSET (rfd2, &readfrom) || FD_ISSET (rem, &readfrom));
+  while ((FD_ISSET (rfd2, &readfrom) || FD_ISSET (rem, &readfrom))
+	 && !end_of_pipe);
 }
 
 void
@@ -748,6 +773,12 @@ sendsig (int sig)
     write (rfd2, &signo, 1);
 }
 
+void
+sigpipe (int sig _GL_UNUSED_PARAMETER)
+{
+  ++end_of_pipe;
+}
+
 #if defined KERBEROS || defined SHISHI
 void
 warning (const char *fmt, ...)
